==> wechat-work-1.0.3-sources.jar: cn/net/wanmo/common/weixin/work/inner/client_api/storage/TicketTokenForAgentConfig.java <==
package cn.net.wanmo.common.weixin.work.inner.client_api.storage;

import cn.net.wanmo.common.pojo.InterfaceResult;
import cn.net.wanmo.common.util.DateUtil;
import cn.net.wanmo.common.util.Threads;
import cn.net.wanmo.common.weixin.work.inner.pojo.log.TokenLog;
import cn.net.wanmo.common.weixin.work.inner.pojo.token.TicketToken;
import cn.net.wanmo.common.weixin.work.inner.client_api.pojo.jsapi_ticket.JsapiTicketReqForAgentConfig;
import cn.net.wanmo.common.weixin.work.inner.client_api.pojo.jsapi_ticket.JsapiTicketResForAgentConfig;
import cn.net.wanmo.common.weixin.work.inner.client_api.util.jsapi_ticket.JsapiTicketUtil;
import cn.net.wanmo.common.weixin.work.inner.pojo.token.AccessToken;
import cn.net.wanmo.common.weixin.work.inner.pojo.Agent;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;


/**
 * 应用票据令牌的定时获取
 */
public class TicketTokenForAgentConfig {
    private static Logger logger = LoggerFactory.getLogger(TicketTokenForAgentConfig.class);


    // ------------------------------- 令牌存储 start -------------------------------------------
    /**
     * 存储令牌 <br/>
     * key: 应用ID  <br/>
     * value: 令牌信息  <br/>
     */
    private static Map<String, TicketToken> ticketMap = new HashMap<>();

    /**
     * 生成 key
     *
     * @param corpId  企业 ID
     * @param agentId 应用ID
     * @return key
     */
    private static String getMapKey(String corpId, String agentId) {
        return corpId + ":" + agentId;
    }

    /**
     * 设置令牌
     *
     * @param corpId                 企业 ID
     * @param agentId                应用ID
     * @param ticketTokenAgentConfig 票据令牌
     */
    public static <TicketTokenAgentConfig extends TicketToken> void putTicket(String corpId, String agentId, TicketTokenAgentConfig ticketTokenAgentConfig) {
        ticketTokenAgentConfig.storeToken(); // 执行自定义存储
        ticketMap.put(getMapKey(corpId, agentId), ticketTokenAgentConfig);
    }

    /**
     * 获取令牌
     *
     * @param corpId  企业 ID
     * @param agentId 应用ID
     * @return 票据令牌
     */
    public static TicketToken getTicket(String corpId, String agentId) {
        return ticketMap.getOrDefault(getMapKey(corpId, agentId), new TicketToken());
    }
    // ------------------------------- 令牌存储 end -------------------------------------------

    // ------------------------------- 令牌初始化 end -------------------------------------------

    /**
     * 获取令牌，并且保存令牌
     *
     * @param agent 应用信息，需包括企业信息和应用令牌
     * @return 接口实际响应的结果
     */
    public static <AgentToken extends AccessToken, JsApiTicket extends TicketToken, JsApiConfigTicket extends TicketToken> InterfaceResult<JsapiTicketResForAgentConfig> initToken(Agent<AgentToken, JsApiTicket, JsApiConfigTicket> agent) {
        String corpId = agent.getCorpId();
        String agentId = agent.getAgentId();
        String logPrevDesc = agent.getLogPrevDesc();
        AgentToken agentToken = agent.getAgentToken();
        return initToken(corpId, agentId, logPrevDesc, agentToken, null);
    }

    /**
     * 获取令牌，并且保存令牌
     *
     * @param agent 应用信息，需包括企业信息和应用令牌
     * @return 接口实际响应的结果
     */
    public static <AgentToken extends AccessToken, JsApiTicket extends TicketToken, JsApiConfigTicket extends TicketToken, TicketTokenAgentConfig extends TicketToken> InterfaceResult<JsapiTicketResForAgentConfig> initToken(Agent<AgentToken, JsApiTicket, JsApiConfigTicket> agent, TicketTokenAgentConfig ticketTokenAgentConfig) {
        String corpId = agent.getCorpId();
        String agentId = agent.getAgentId();
        String logPrevDesc = agent.getLogPrevDesc();
        AgentToken agentToken = agent.getAgentToken();
        return initToken(corpId, agentId, logPrevDesc, agentToken, ticketTokenAgentConfig);
    }


    /**
     * 获取令牌，并且保存令牌
     *
     * @param corpId      企业 ID
     * @param agentId     应用ID
     * @param logPrevDesc 日志前缀描述
     * @param agentToken  应用令牌
     * @return 接口实际响应的结果
     */
    public static <AgentToken extends AccessToken> InterfaceResult<JsapiTicketResForAgentConfig> initToken(String corpId, String agentId, String logPrevDesc, AgentToken agentToken) {

        return initToken(corpId, agentId, logPrevDesc, agentToken, null);
    }

    /**
     * 获取令牌，并且保存令牌
     *
     * @param corpId                 企业 ID
     * @param agentId                应用ID
     * @param logPrevDesc            日志前缀描述
     * @param agentToken             应用令牌
     * @param ticketTokenAgentConfig 扩展的自定义 ticket 对象，重写 storeToken() 方法，支持自定义存储令牌
     * @return 接口实际响应的结果
     */
    public static <AgentToken extends AccessToken, TicketTokenAgentConfig extends TicketToken> InterfaceResult<JsapiTicketResForAgentConfig> initToken(String corpId, String agentId, String logPrevDesc, AgentToken agentToken, TicketTokenAgentConfig ticketTokenAgentConfig) {
        JsapiTicketReqForAgentConfig req = JsapiTicketReqForAgentConfig.build();
        String accessToken = agentToken.takeToken();
        InterfaceResult<JsapiTicketResForAgentConfig> result = JsapiTicketUtil.getJsapiTicketForAgentConfig(logPrevDesc, accessToken, req);

        TicketToken ticketToken = new TicketToken();
        ticketToken.setCorpId(corpId);
        ticketToken.setAgentId(agentId);
        ticketToken.setTokenType(TicketToken.TokenType.agent_config);
        ticketToken.setCode(result.getCode());
        ticketToken.setMsg(result.getMsg());
        ticketToken.setResTime(DateUtil.nowLong());

        if (result.isSuccess()) {
            JsapiTicketResForAgentConfig data = result.getData();
            ticketToken.setValue(data.getTicket());
            ticketToken.setExpiresIn(data.getExpiresIn());
            ticketToken.setResTime(data.getResTime());
        }

        if (Objects.isNull(ticketTokenAgentConfig)) {
            putTicket(corpId, agentId, ticketToken);
            logger.debug("企业应用ID: {}, 执行默认存储应用 jsApiConfig 票据令牌完成 ...", agentId);
        } else {
            { // 赋值操作
                ticketTokenAgentConfig.setCorpId(ticketToken.getCorpId());
                ticketTokenAgentConfig.setAgentId(ticketToken.getAgentId());
                ticketTokenAgentConfig.setTokenType(ticketToken.getTokenType());
                ticketTokenAgentConfig.setCode(ticketToken.getCode());
                ticketTokenAgentConfig.setMsg(ticketToken.getMsg());
                ticketTokenAgentConfig.setValue(ticketToken.getValue());
                ticketTokenAgentConfig.setExpiresIn(ticketToken.getExpiresIn());
                ticketTokenAgentConfig.setResTime(ticketToken.getResTime());
            }
            putTicket(corpId, agentId, ticketTokenAgentConfig);
            logger.debug("企业应用ID: {}, 执行自定义存储应用 jsApiConfig 票据令牌完成 ...", agentId);
        }

        { // 内存中记录日志
            TokenLog.add(corpId, agentId, ticketToken.getTokenType().getLabel(), ticketToken, TokenLog.ticketConfigTokens);
        }

        return result;
    }
    // ------------------------------- 令牌初始化 end -------------------------------------------


    // ------------------------------- 令牌线程 start -------------------------------------------

    /**
     * 自动定时获取 令牌
     *
     * @param agent 应用信息，需包括企业信息和应用令牌
     */
    public static <AgentToken extends AccessToken, JsApiTicket extends TicketToken, JsApiConfigTicket extends TicketToken> void runThreadInitToken(Agent<AgentToken, JsApiTicket, JsApiConfigTicket> agent) {
        String corpId = agent.getCorpId();
        String agentId = agent.getAgentId();
        String logPrevDesc = agent.getLogPrevDesc();
        AgentToken agentToken = agent.getAgentToken();

        runThreadInitToken(corpId, agentId, logPrevDesc, agentToken, null, null, null);
    }

    /**
     * 自动定时获取 令牌
     *
     * @param agent 应用信息，需包括企业信息和应用令牌
     */
    public static <AgentToken extends AccessToken, JsApiTicket extends TicketToken, JsApiConfigTicket extends TicketToken> void runThreadInitToken(Agent<AgentToken, JsApiTicket, JsApiConfigTicket> agent, Integer residueSecond, Integer waitSecond) {
        String corpId = agent.getCorpId();
        String agentId = agent.getAgentId();
        String logPrevDesc = agent.getLogPrevDesc();
        AgentToken agentToken = agent.getAgentToken();

        runThreadInitToken(corpId, agentId, logPrevDesc, agentToken, residueSecond, waitSecond, null);
    }


    /**
     * 自动定时获取 令牌
     *
     * @param agent                  应用信息，需包括企业信息和应用令牌
     * @param ticketTokenAgentConfig 扩展的自定义 ticket 对象，重写 storeToken() 方法，支持自定义存储令牌
     */
    public static <AgentToken extends AccessToken, JsApiTicket extends TicketToken, JsApiConfigTicket extends TicketToken, TicketTokenAgentConfig extends TicketToken> void runThreadInitToken(Agent<AgentToken, JsApiTicket, JsApiConfigTicket> agent, TicketTokenAgentConfig ticketTokenAgentConfig) {
        String corpId = agent.getCorpId();
        String agentId = agent.getAgentId();
        String logPrevDesc = agent.getLogPrevDesc();
        AgentToken agentToken = agent.getAgentToken();

        runThreadInitToken(corpId, agentId, logPrevDesc, agentToken, null, null, ticketTokenAgentConfig);
    }

    /**
     * 自动定时获取 令牌
     *
     * @param agent                  应用信息，需包括企业信息和应用令牌
     * @param residueSecond          成功后，剩余多少时间后重新获取（默认 30秒）
     * @param waitSecond             失败后，等待多少时间后重新获取（默认 60秒）
     * @param ticketTokenAgentConfig 扩展的自定义 ticket 对象，重写 storeToken() 方法，支持自定义存储令牌
     */
    public static <AgentToken extends AccessToken, JsApiTicket extends TicketToken, JsApiConfigTicket extends TicketToken, TicketTokenAgentConfig extends TicketToken> void runThreadInitToken(Agent<AgentToken, JsApiTicket, JsApiConfigTicket> agent, Integer residueSecond, Integer waitSecond, TicketTokenAgentConfig ticketTokenAgentConfig) {
        String corpId = agent.getCorpId();
        String agentId = agent.getAgentId();
        String logPrevDesc = agent.getLogPrevDesc();
        AgentToken agentToken = agent.getAgentToken();

        runThreadInitToken(corpId, agentId, logPrevDesc, agentToken, residueSecond, waitSecond, ticketTokenAgentConfig);
    }

    /**
     * 自动定时获取 令牌
     *
     * @param corpId      企业 ID
     * @param agentId     应用 ID
     * @param logPrevDesc 日志前缀描述
     * @param agentToken  应用令牌
     */
    public static <AgentToken extends AccessToken> void runThreadInitToken(String corpId, String agentId, String logPrevDesc, AgentToken agentToken) {
        runThreadInitToken(corpId, agentId, logPrevDesc, agentToken, null, null, null);
    }

    /**
     * 自动定时获取 令牌
     *
     * @param corpId        企业 ID
     * @param agentId       应用 ID
     * @param logPrevDesc   日志前缀描述
     * @param agentToken    应用令牌
     * @param residueSecond 成功后，剩余多少时间后重新获取（默认 30秒）
     * @param waitSecond    失败后，等待多少时间后重新获取（默认 60秒）
     */
    public static <AgentToken extends AccessToken> void runThreadInitToken(String corpId, String agentId, String logPrevDesc, AgentToken agentToken, Integer residueSecond, Integer waitSecond) {
        runThreadInitToken(corpId, agentId, logPrevDesc, agentToken, residueSecond, waitSecond, null);
    }


    /**
     * 自动定时获取 令牌
     *
     * @param corpId                 企业 ID
     * @param agentId                应用 ID
     * @param logPrevDesc            日志前缀描述
     * @param agentToken             应用令牌
     * @param ticketTokenAgentConfig 扩展的自定义 ticket 对象，重写 storeToken() 方法，支持自定义存储令牌
     */
    public static <AgentToken extends AccessToken, TicketTokenAgentConfig extends TicketToken> void runThreadInitToken(String corpId, String agentId, String logPrevDesc, AgentToken agentToken, TicketTokenAgentConfig ticketTokenAgentConfig) {
        runThreadInitToken(corpId, agentId, logPrevDesc, agentToken, null, null, ticketTokenAgentConfig);
    }

    /**
     * 自动定时获取 令牌
     *
     * @param corpId                 企业 ID
     * @param agentId                应用 ID
     * @param logPrevDesc            日志前缀描述
     * @param agentToken             应用令牌
     * @param residueSecond          成功后，剩余多少时间后重新获取（默认 30秒）
     * @param waitSecond             失败后，等待多少时间后重新获取（默认 60秒）
     * @param ticketTokenAgentConfig 扩展的自定义 ticket 对象，重写 storeToken() 方法，支持自定义存储令牌
     */
    public static <AgentToken extends AccessToken, TicketTokenAgentConfig extends TicketToken> void runThreadInitToken(String corpId, String agentId, String logPrevDesc, AgentToken agentToken, Integer residueSecond, Integer waitSecond, TicketTokenAgentConfig ticketTokenAgentConfig) {
        new Thread(new Runnable() {
            @Override
            public void run() {
                while (true) {
                    try {
                        InterfaceResult<JsapiTicketResForAgentConfig> result = initToken(corpId, agentId, logPrevDesc, agentToken, ticketTokenAgentConfig);
                        if (result.isSuccess()) { // 令牌获取成功
                            String ticketValue = result.getData().getTicket();
                            Integer expiresIn = result.getData().getExpiresIn();

                            logger.debug("企业应用ID: {}, 定时任务获取 应用配置 jsApiConfig 令牌 成功，expiresIn = {}， ticket = {}", agentId, expiresIn, ticketValue);
                            Threads.sleep((expiresIn - (residueSecond == null ? 30 : residueSecond)) * 1000); // 休眠到 有效期剩余 30秒
                        } else { // 令牌获取失败
                            logger.error("企业应用ID: {}, 定时任务获取 应用配置 jsApiConfig 令牌 失败, errCode:{} errMsg:{}", agentId, result.getCode(), result.getMsg());
                            Threads.sleep((waitSecond == null ? 60 : waitSecond) * 1000); // 休眠到 60秒
                        }
                    } catch (Exception e) {
                        logger.error("企业应用ID: " + agentId + ", 定时任务获取 应用配置jsapi令牌 异常", e);
                        Threads.sleep((waitSecond == null ? 60 : waitSecond) * 1000); // 休眠到 60秒
                    }
                }
            }
        }).start();
    }
    // ------------------------------- 令牌线程 end -------------------------------------------
}

==> wechat-work-1.0.4-sources.jar: cn/net/wanmo/common/weixin/work/inner/client_api/storage/TicketTokenForAgentConfig.java <==
package cn.net.wanmo.common.weixin.work.inner.client_api.storage;

import cn.net.wanmo.common.result.InterfaceResult;
import cn.net.wanmo.common.util.DateUtil;
import cn.net.wanmo.common.util.Threads;
import cn.net.wanmo.common.weixin.work.inner.pojo.log.TokenLog;
import cn.net.wanmo.common.weixin.work.inner.pojo.token.TicketToken;
import cn.net.wanmo.common.weixin.work.inner.client_api.pojo.jsapi_ticket.JsapiTicketReqForAgentConfig;
import cn.net.wanmo.common.weixin.work.inner.client_api.pojo.jsapi_ticket.JsapiTicketResForAgentConfig;
import cn.net.wanmo.common.weixin.work.inner.client_api.util.jsapi_ticket.JsapiTicketUtil;
import cn.net.wanmo.common.weixin.work.inner.pojo.token.AccessToken;
import cn.net.wanmo.common.weixin.work.inner.pojo.Agent;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;


/**
 * 应用票据令牌的定时获取
 */
public class TicketTokenForAgentConfig {
    private static Logger logger = LoggerFactory.getLogger(TicketTokenForAgentConfig.class);


    // ------------------------------- 令牌存储 start -------------------------------------------
    /**
     * 存储令牌 <br/>
     * key: 应用ID  <br/>
     * value: 令牌信息  <br/>
     */
    private static Map<String, TicketToken> ticketMap = new HashMap<>();

    /**
     * 生成 key
     *
     * @param corpId  企业 ID
     * @param agentId 应用ID
     * @return key
     */
    private static String getMapKey(String corpId, String agentId) {
        return corpId + ":" + agentId;
    }

    /**
     * 设置令牌
     *
     * @param corpId                 企业 ID
     * @param agentId                应用ID
     * @param ticketTokenAgentConfig 票据令牌
     */
    public static <TicketTokenAgentConfig extends TicketToken> void putTicket(String corpId, String agentId, TicketTokenAgentConfig ticketTokenAgentConfig) {
        ticketTokenAgentConfig.storeToken(); // 执行自定义存储
        ticketMap.put(getMapKey(corpId, agentId), ticketTokenAgentConfig);
    }

    /**
     * 获取令牌
     *
     * @param corpId  企业 ID
     * @param agentId 应用ID
     * @return 票据令牌
     */
    public static TicketToken getTicket(String corpId, String agentId) {
        return ticketMap.getOrDefault(getMapKey(corpId, agentId), new TicketToken());
    }
    // ------------------------------- 令牌存储 end -------------------------------------------

    // ------------------------------- 令牌初始化 end -------------------------------------------

    /**
     * 获取令牌，并且保存令牌
     *
     * @param agent 应用信息，需包括企业信息和应用令牌
     * @return 接口实际响应的结果
     */
    public static <AgentToken extends AccessToken, JsApiTicket extends TicketToken, JsApiConfigTicket extends TicketToken> InterfaceResult<JsapiTicketResForAgentConfig> initToken(Agent<AgentToken, JsApiTicket, JsApiConfigTicket> agent) {
        String corpId = agent.getCorpId();
        String agentId = agent.getAgentId();
        String logPrevDesc = agent.getLogPrevDesc();
        AgentToken agentToken = agent.getAgentToken();
        return initToken(corpId, agentId, logPrevDesc, agentToken, null);
    }

    /**
     * 获取令牌，并且保存令牌
     *
     * @param agent 应用信息，需包括企业信息和应用令牌
     * @return 接口实际响应的结果
     */
    public static <AgentToken extends AccessToken, JsApiTicket extends TicketToken, JsApiConfigTicket extends TicketToken, TicketTokenAgentConfig extends TicketToken> InterfaceResult<JsapiTicketResForAgentConfig> initToken(Agent<AgentToken, JsApiTicket, JsApiConfigTicket> agent, TicketTokenAgentConfig ticketTokenAgentConfig) {
        String corpId = agent.getCorpId();
        String agentId = agent.getAgentId();
        String logPrevDesc = agent.getLogPrevDesc();
        AgentToken agentToken = agent.getAgentToken();
        return initToken(corpId, agentId, logPrevDesc, agentToken, ticketTokenAgentConfig);
    }


    /**
     * 获取令牌，并且保存令牌
     *
     * @param corpId      企业 ID
     * @param agentId     应用ID
     * @param logPrevDesc 日志前缀描述
     * @param agentToken  应用令牌
     * @return 接口实际响应的结果
     */
    public static <AgentToken extends AccessToken> InterfaceResult<JsapiTicketResForAgentConfig> initToken(String corpId, String agentId, String logPrevDesc, AgentToken agentToken) {

        return initToken(corpId, agentId, logPrevDesc, agentToken, null);
    }

    /**
     * 获取令牌，并且保存令牌
     *
     * @param corpId                 企业 ID
     * @param agentId                应用ID
     * @param logPrevDesc            日志前缀描述
     * @param agentToken             应用令牌
     * @param ticketTokenAgentConfig 扩展的自定义 ticket 对象，重写 storeToken() 方法，支持自定义存储令牌
     * @return 接口实际响应的结果
     */
    public static <AgentToken extends AccessToken, TicketTokenAgentConfig extends TicketToken> InterfaceResult<JsapiTicketResForAgentConfig> initToken(String corpId, String agentId, String logPrevDesc, AgentToken agentToken, TicketTokenAgentConfig ticketTokenAgentConfig) {
        JsapiTicketReqForAgentConfig req = JsapiTicketReqForAgentConfig.build();
        String accessToken = agentToken.takeToken();
        InterfaceResult<JsapiTicketResForAgentConfig> result = JsapiTicketUtil.getJsapiTicketForAgentConfig(logPrevDesc, accessToken, req);

        TicketToken ticketToken = new TicketToken();
        ticketToken.setCorpId(corpId);
        ticketToken.setAgentId(agentId);
        ticketToken.setTokenType(TicketToken.TokenType.jsapi_ticket_config);
        ticketToken.setCode(result.getCode());
        ticketToken.setMsg(result.getMsg());
        ticketToken.setResTime(DateUtil.nowLong());

        if (result.isSuccess()) {
            JsapiTicketResForAgentConfig data = result.getData();
            ticketToken.setValue(data.getTicket());
            ticketToken.setExpiresIn(data.getExpiresIn());
            ticketToken.setResTime(data.getResTime());
        }

        if (Objects.isNull(ticketTokenAgentConfig)) {
            putTicket(corpId, agentId, ticketToken);
            logger.debug("企业应用ID: {}, 执行默认存储应用 jsApiConfig 票据令牌完成 ...", agentId);
        } else {
            { // 赋值操作
                ticketTokenAgentConfig.setCorpId(ticketToken.getCorpId());
                ticketTokenAgentConfig.setAgentId(ticketToken.getAgentId());
                ticketTokenAgentConfig.setTokenType(ticketToken.getTokenType());
                ticketTokenAgentConfig.setCode(ticketToken.getCode());
                ticketTokenAgentConfig.setMsg(ticketToken.getMsg());
                ticketTokenAgentConfig.setValue(ticketToken.getValue());
                ticketTokenAgentConfig.setExpiresIn(ticketToken.getExpiresIn());
                ticketTokenAgentConfig.setResTime(ticketToken.getResTime());
            }
            putTicket(corpId, agentId, ticketTokenAgentConfig);
            logger.debug("企业应用ID: {}, 执行自定义存储应用 jsApiConfig 票据令牌完成 ...", agentId);
        }

        { // 内存中记录日志
            TokenLog.add(corpId, agentId, ticketToken.getTokenType().getLabel(), ticketToken, TokenLog.ticketConfigTokens);
        }

        return result;
    }
    // ------------------------------- 令牌初始化 end -------------------------------------------


    // ------------------------------- 令牌线程 start -------------------------------------------

    /**
     * 自动定时获取 令牌
     *
     * @param agent 应用信息，需包括企业信息和应用令牌
     */
    public static <AgentToken extends AccessToken, JsApiTicket extends TicketToken, JsApiConfigTicket extends TicketToken> void runThreadInitToken(Agent<AgentToken, JsApiTicket, JsApiConfigTicket> agent) {
        String corpId = agent.getCorpId();
        String agentId = agent.getAgentId();
        String logPrevDesc = agent.getLogPrevDesc();
        AgentToken agentToken = agent.getAgentToken();

        runThreadInitToken(corpId, agentId, logPrevDesc, agentToken, null, null, null);
    }

    /**
     * 自动定时获取 令牌
     *
     * @param agent 应用信息，需包括企业信息和应用令牌
     */
    public static <AgentToken extends AccessToken, JsApiTicket extends TicketToken, JsApiConfigTicket extends TicketToken> void runThreadInitToken(Agent<AgentToken, JsApiTicket, JsApiConfigTicket> agent, Integer residueSecond, Integer waitSecond) {
        String corpId = agent.getCorpId();
        String agentId = agent.getAgentId();
        String logPrevDesc = agent.getLogPrevDesc();
        AgentToken agentToken = agent.getAgentToken();

        runThreadInitToken(corpId, agentId, logPrevDesc, agentToken, residueSecond, waitSecond, null);
    }


    /**
     * 自动定时获取 令牌
     *
     * @param agent                  应用信息，需包括企业信息和应用令牌
     * @param ticketTokenAgentConfig 扩展的自定义 ticket 对象，重写 storeToken() 方法，支持自定义存储令牌
     */
    public static <AgentToken extends AccessToken, JsApiTicket extends TicketToken, JsApiConfigTicket extends TicketToken, TicketTokenAgentConfig extends TicketToken> void runThreadInitToken(Agent<AgentToken, JsApiTicket, JsApiConfigTicket> agent, TicketTokenAgentConfig ticketTokenAgentConfig) {
        String corpId = agent.getCorpId();
        String agentId = agent.getAgentId();
        String logPrevDesc = agent.getLogPrevDesc();
        AgentToken agentToken = agent.getAgentToken();

        runThreadInitToken(corpId, agentId, logPrevDesc, agentToken, null, null, ticketTokenAgentConfig);
    }

    /**
     * 自动定时获取 令牌
     *
     * @param agent                  应用信息，需包括企业信息和应用令牌
     * @param residueSecond          成功后，剩余多少时间后重新获取（默认 30秒）
     * @param waitSecond             失败后，等待多少时间后重新获取（默认 60秒）
     * @param ticketTokenAgentConfig 扩展的自定义 ticket 对象，重写 storeToken() 方法，支持自定义存储令牌
     */
    public static <AgentToken extends AccessToken, JsApiTicket extends TicketToken, JsApiConfigTicket extends TicketToken, TicketTokenAgentConfig extends TicketToken> void runThreadInitToken(Agent<AgentToken, JsApiTicket, JsApiConfigTicket> agent, Integer residueSecond, Integer waitSecond, TicketTokenAgentConfig ticketTokenAgentConfig) {
        String corpId = agent.getCorpId();
        String agentId = agent.getAgentId();
        String logPrevDesc = agent.getLogPrevDesc();
        AgentToken agentToken = agent.getAgentToken();

        runThreadInitToken(corpId, agentId, logPrevDesc, agentToken, residueSecond, waitSecond, ticketTokenAgentConfig);
    }

    /**
     * 自动定时获取 令牌
     *
     * @param corpId      企业 ID
     * @param agentId     应用 ID
     * @param logPrevDesc 日志前缀描述
     * @param agentToken  应用令牌
     */
    public static <AgentToken extends AccessToken> void runThreadInitToken(String corpId, String agentId, String logPrevDesc, AgentToken agentToken) {
        runThreadInitToken(corpId, agentId, logPrevDesc, agentToken, null, null, null);
    }

    /**
     * 自动定时获取 令牌
     *
     * @param corpId        企业 ID
     * @param agentId       应用 ID
     * @param logPrevDesc   日志前缀描述
     * @param agentToken    应用令牌
     * @param residueSecond 成功后，剩余多少时间后重新获取（默认 30秒）
     * @param waitSecond    失败后，等待多少时间后重新获取（默认 60秒）
     */
    public static <AgentToken extends AccessToken> void runThreadInitToken(String corpId, String agentId, String logPrevDesc, AgentToken agentToken, Integer residueSecond, Integer waitSecond) {
        runThreadInitToken(corpId, agentId, logPrevDesc, agentToken, residueSecond, waitSecond, null);
    }


    /**
     * 自动定时获取 令牌
     *
     * @param corpId                 企业 ID
     * @param agentId                应用 ID
     * @param logPrevDesc            日志前缀描述
     * @param agentToken             应用令牌
     * @param ticketTokenAgentConfig 扩展的自定义 ticket 对象，重写 storeToken() 方法，支持自定义存储令牌
     */
    public static <AgentToken extends AccessToken, TicketTokenAgentConfig extends TicketToken> void runThreadInitToken(String corpId, String agentId, String logPrevDesc, AgentToken agentToken, TicketTokenAgentConfig ticketTokenAgentConfig) {
        runThreadInitToken(corpId, agentId, logPrevDesc, agentToken, null, null, ticketTokenAgentConfig);
    }

    /**
     * 自动定时获取 令牌
     *
     * @param corpId                 企业 ID
     * @param agentId                应用 ID
     * @param logPrevDesc            日志前缀描述
     * @param agentToken             应用令牌
     * @param residueSecond          成功后，剩余多少时间后重新获取（默认 30秒）
     * @param waitSecond             失败后，等待多少时间后重新获取（默认 60秒）
     * @param ticketTokenAgentConfig 扩展的自定义 ticket 对象，重写 storeToken() 方法，支持自定义存储令牌
     */
    public static <AgentToken extends AccessToken, TicketTokenAgentConfig extends TicketToken> void runThreadInitToken(String corpId, String agentId, String logPrevDesc, AgentToken agentToken, Integer residueSecond, Integer waitSecond, TicketTokenAgentConfig ticketTokenAgentConfig) {
        new Thread(new Runnable() {
            @Override
            public void run() {
                while (true) {
                    try {
                        InterfaceResult<JsapiTicketResForAgentConfig> result = initToken(corpId, agentId, logPrevDesc, agentToken, ticketTokenAgentConfig);
                        if (result.isSuccess()) { // 令牌获取成功
                            String ticketValue = result.getData().getTicket();
                            Integer expiresIn = result.getData().getExpiresIn();

                            logger.debug("企业应用ID: {}, 定时任务获取 应用配置 jsApiConfig 令牌 成功，expiresIn = {}， ticket = {}", agentId, expiresIn, ticketValue);
                            Threads.sleep((expiresIn - (residueSecond == null ? 30 : residueSecond)) * 1000); // 休眠到 有效期剩余 30秒
                        } else { // 令牌获取失败
                            logger.error("企业应用ID: {}, 定时任务获取 应用配置 jsApiConfig 令牌 失败, errCode:{} errMsg:{}", agentId, result.getCode(), result.getMsg());
                            Threads.sleep((waitSecond == null ? 60 : waitSecond) * 1000); // 休眠到 60秒后，再次获取
                        }
                    } catch (Exception e) {
                        logger.error("企业应用ID: " + agentId + ", 定时任务获取 应用配置jsapi令牌 异常", e);
                        Threads.sleep((waitSecond == null ? 60 : waitSecond) * 1000); // 休眠到 60秒后，再次获取
                    }
                }
            }
        }).start();
    }
    // ------------------------------- 令牌线程 end -------------------------------------------
}
